@@ -1,9 +1,96 @@
 import enum
 from collections.abc import MutableMapping
 from configparser import ConfigParser, NoOptionError
+from json.decoder import JSONDecodeError
 import os
 import logging
 import re
+import json
+from typing import List
+
+
+class ObjectValue:
+
+    def __eq__(self, other):
+        if isinstance(other, ObjectValue):
+            return self._name == other._name and self._value == other._value
+        return False
+
+    @classmethod
+    def fromString(cls, str_val):
+        """
+        Creates a ObjectValue object from a given string
+        string format must be '{object_name}{object}'
+        Example: 'X{"key", "value"}'
+        """
+        obj_val = cls.parse_string(str_val)
+        return cls(name=obj_val[0], value=obj_val[1])
+
+    def parse_string(str_val: str) -> List[str]:
+        """
+        Parses object value string
+        returns: List[str]
+            [0] object_name
+            [1] object_string (json)
+        """
+        try:
+            obj_val_rgx = r"(\w*)([{].*?[}])$"
+            result = re.search(obj_val_rgx, str_val)
+            object_name = result.group(1)
+            object_string = result.group(2)
+            return [object_name, object_string]
+        except AttributeError:
+            raise AttributeError(f"Cant find an object in the stirng: {str_val}")
+
+    @classmethod
+    def is_object_value(cls, str_val) -> bool:
+        """
+        Checks if a given str_val is an object value
+        """
+        try:
+            result = cls.parse_string(str_val)
+            return bool(result[1])
+        except:
+            return False
+
+    def __init__(self, name="", value="{}") -> None:
+        """
+        string representation of the object. Value must be parsable to a valid json object {}
+        It must containing the surrounding curly brackets. Value cannot be an array, only an object.
+        Example: {key1: 42, key3: "Foo"}
+        """
+        self._name = name
+
+        self._value = value
+
+    def __str__(self) -> str:
+        return f"{self._name}{self._value}"
+
+    @property
+    def as_dict(self):
+        return json.loads(self._value)
+
+    @as_dict.setter
+    def as_dict(self, val):
+        self._value = json.dumps(val)
+
+    def add(self, key, value, override=True):
+        """
+        add key/value pair to self._value depending on override flag.
+        If override False and value exist return false otherwise true. No error
+        """
+        dict_val = json.loads(self._value)
+        if override:
+            dict_val[key] = value
+            self.as_dict = dict_val
+            return True
+        else:
+            if key in self.as_dict:
+                return False
+            else:
+                dict_val[key] = value
+                self.as_dict = dict_val
+                return True
 
 
 class OppParser(ConfigParser):
@@ -36,7 +123,7 @@ class OppParser(ConfigParser):
 
         with open(output_path, "w") as f:
             f.writelines(file_content)
-        return  file_content
+        return file_content
 
     def read(self, filenames, encoding=None):
 
@@ -57,6 +144,7 @@ class OppParser(ConfigParser):
         file_contents = list()
 
         for f in filenames:
+            self.ini_root_path = os.path.dirname(os.path.abspath(f))
             file_content = self.get_file_content_recursively(f)
             file_contents.append(file_content)
 
@@ -128,6 +216,7 @@ class OppParser(ConfigParser):
                 file_content = file_content.replace(
                     replace_string, self.get_file_content_recursively(inc_file_name)
                 )
+                self.current_dir = self.ini_root_path
 
         return file_content
 
@@ -269,7 +358,7 @@ class OppConfigFileBase(MutableMapping):
         return self._section_hierarchy
 
     def resolve_path(self, key):
-        val =  self.__getitem__(key)
+        val = self.__getitem__(key)
         p = re.compile('absFilePath\("(.*)"\)')
         m = p.match(val)
         if m:
@@ -277,7 +366,6 @@ class OppConfigFileBase(MutableMapping):
         else:
             return os.path.join(self.base_path, val.strip('"'))
 
-
     def read(self):
         self.read()
         pass
@@ -334,6 +422,11 @@ class OppConfigFileBase(MutableMapping):
                     return p.get_config_for_option(option)
         return None
 
+    def _is_json_list(self, _str) -> bool:
+        # todo: replace with is object value
+        obj_rgx = r"[\[].*?[\]]$"
+        return bool(re.search(obj_rgx, str(_str)))
+
     def _has_section_(self, sec):
         """
         True if section exist in the configuration. Note: Returns also True even if given section is not
@@ -346,12 +439,24 @@ class OppConfigFileBase(MutableMapping):
         Search for key in local configuration
         """
         try:
-            return self._root.get(self._sec, k)
+            item = self._root.get(self._sec, k)
+            if "\n" in item:
+                item = item.replace("\n", " ")
+            if ObjectValue.is_object_value(item):
+                return ObjectValue().fromString(item)
+            if self._is_json_list(item):
+                return json.loads(item)
+            else:
+                return self._root.get(self._sec, k)
         except NoOptionError:
             if default is not None:
                 return default
             else:
                 raise KeyError(f"key not found. Key: {k}")
+        except KeyError:
+            return self._root.get(self._sec, k)
+        except json.JSONDecodeError as e:
+            raise KeyError(f"key '{k}' has invalid json value: '{item}' ")
 
     def _set_local(self, k, v):
         """
@@ -481,8 +586,7 @@ class OppConfigFile(OppConfigFileBase):
 
 
 if __name__ == "__main__":
-
     OppParser.resolve_includes(
         "/home/vm-sts/repos/crownet/crownet/simulations/mucFreiheitLte/omnetpp.ini",
-        output_path= "/home/vm-sts/repos/crownet/crownet/simulations/mucFreiheitLte/omnetpp_merged.ini"
-    )
\ No newline at end of file
+        output_path="/home/vm-sts/repos/crownet/crownet/simulations/mucFreiheitLte/omnetpp_merged.ini"
+    )
